@@ -13,6 +13,8 @@ class RestAPI(object):
 			firebase = Firebase(...)
 	"""
 	def __init__(self,
+		# remove subscriptions when a user is subscribed to more then subscription.
+		remove_double_subscriptions=True,
 		# Pass either the firebase credentials or initialzed firebase object.
 		# 	the firebase credentials.
 		firebase_credentials=None,
@@ -79,6 +81,9 @@ class RestAPI(object):
 			},
 		}
 
+		# arguments.
+		self.remove_double_subscriptions = remove_double_subscriptions
+
 		# system variables.
 		self.uid_api_keys = {
 			"$uid":"$api_key..."
@@ -267,8 +272,7 @@ class RestAPI(object):
 
 		# error.
 		except Exception as e:
-			print("Error:",e)
-			return r3sponse.error_response("Failed to retrieve the subscriptions.")
+			return r3sponse.error_response(f"Failed to retrieve the subscriptions, error: {e}.")
 
 		#
 	def create_subscription(self):
@@ -314,39 +318,49 @@ class RestAPI(object):
 			return r3sponse.error_response(f"Failed to cancel the subscription for user [{uid}] from plan [{plan}].")
 
 		#
-	def synchronize_users(self):
+	def synchronize_users(self, silent=False):
 		"""
 			Checks all users & inserts new data.
 			Maps the api keys per subscription per plan.
 		"""
 
+		# logs.
+		log_level = 1
+		if silent == False: 
+			log_level = 0
+			print(f"Synchronizing all users.")
+
 		# get subs.
 		response = self.get_subscriptions()
 		if response["error"] != None: return response
 		subscriptions = response["subscriptions"]
+		if silent == False: print(f"Found {len(subscriptions)} subscriptions.")
 
 		# check-remove double subscriptions.
-		rank, plan, cancel_subs = None, None, {}
-		for uid, plans in subscriptions.items():
-			for plan, info in plans.items():
-				if info["plan_active_status"]:
-					response = self.__get_plan_by_plan_id__(info["plan_id"])
-					if response["error"] != None: return response
-					l_plan = response["plan"]
-					l_rank = self.__plans__[l_plan]["rank"]
-					if plan == None:
-						rank, plan = int(l_rank), str(l_plan)
-					else:
-						if int(l_rank) > int(rank):
-							try:cancel_subs[uid]
-							except KeyError: cancel_subs[uid] = []
-							if plan not in ["free"]:
-								cancel_subs[uid].append(plan)
+		if self.remove_double_subscriptions:
+			rank, plan, cancel_subs = None, None, {}
+			for uid, plans in subscriptions.items():
+				for plan, info in plans.items():
+					if info["plan_active_status"]:
+						response = self.__get_plan_by_plan_id__(info["plan_id"])
+						if response["error"] != None: return response
+						l_plan = response["plan"]
+						l_rank = self.__plans__[l_plan]["rank"]
+						if plan == None:
 							rank, plan = int(l_rank), str(l_plan)
-		for uid, plans in cancel_subs.items():
-			for plan in plans:
-				response = self.cancel_subscription(uid=uid, plan=plan)
-				if response["error"] != None: return response
+						else:
+							if int(l_rank) > int(rank):
+								try:cancel_subs[uid]
+								except KeyError: cancel_subs[uid] = []
+								if plan not in ["free"]:
+									cancel_subs[uid].append(plan)
+								rank, plan = int(l_rank), str(l_plan)
+					if silent == False: print(f"Found subscription {uid}:{l_plan}, active: {info['plan_active_status']}.")	
+			for uid, plans in cancel_subs.items():
+				for plan in plans:
+					if silent == False: print(f"Canceling double subscription {uid}:{l_plan}.")	
+					response = self.cancel_subscription(uid=uid, plan=plan)
+					if response["error"] != None: return response
 		# reset.
 		for plan in list(self.__plans__.keys()):
 			self.__plans__[plan]["api_keys"] = []
@@ -382,14 +396,14 @@ class RestAPI(object):
 			api_key = data["api_key"]
 			try: self.__plans__[plan]
 			except KeyError: 
-				return r3sponse.error_response(f"Failed to synchronize the users, user [{uid}] has an unknown membership [{plan}].")
+				return r3sponse.error_response(f"Failed to synchronize the users, user [{uid}] has an unknown membership [{plan}].", log_level=log_level, save=True)
 			self.__plans__[plan]["api_keys"].append(api_key)
 
 			# map uid api key.
 			self.uid_api_keys[uid] = api_key
 
 		# success response.
-		return r3sponse.success_response("Successfully synchronized the users.")
+		return r3sponse.success_response("Successfully synchronized the users.", log_level=log_level)
 
 		#
 	# system functions.
